"""
The Slogan Log Analyser
HTML interfacing utilities
"""

import slogan, slogan.skins
[false, true]= range(2)
import os, os.path, string, re

## Initialisation
## =============================================================================

iconType= '.gif'

def initialise(set):
  global config, skin
  import slogan.settings
  config= slogan.settings.settings[set][1]

  if config['skin']=='default':
    skin= slogan.skins.Skin()
  else:
    if config['skin'] in slogan.skins.__all__:
      __import__('slogan.skins.'+config['skin'])
      skin= getattr(slogan.skins, config['skin']).Skin()
    else:
      raise NameError, 'Skin "'+config['skin']+'" not known'

## Skin utils
## =============================================================================

views= ['list', 'tree', 'bar']

## pageType, outputPage -- just pass to the skin

def pageType(pars):
  return skin.pageType(pars)

def outputPage(stream, pars):
  return skin.outputPage(stream, pars)

## Type mapping

sortTypes= {
  'ascii': slogan.sortType_ascii,
  'hits': slogan.sortType_hits,
  '-hits': -slogan.sortType_hits
}

def sortType(n):
  return sortTypes.get(n, slogan.sortType_ascii)

def sortName(t):
  for n in sortTypes.keys():
    if sortTypes[n]==t:
      return n
  return 'ascii'

displayTypes= [
  'list',
  'tree',
  'bar',
  'pie'
]

def displayType(n):
  if n in displayTypes:
    return n
  else:
    return 'list'

## HTML utils
## =============================================================================

## HTML/XML doctype names

doctypes= {
  '2.0': '<!DOCTYPE html PUBLIC "-//IETF//DTD HTML 2.0//EN">\n',
  '3.2': '<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">\n',
  '4.0s': '<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.0 Strict//EN">\n',
  '4.0t': '<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">\n',
  '4.0f': '<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.0 Frameset//EN">\n',
  'x1.0s': '<?xml version="1.0" encoding="UTF-8"?>\n<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN">\n',
  'x1.0t': '<?xml version="1.0" encoding="UTF-8"?>\n<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN">\n',
  'x1.0f': '<?xml version="1.0" encoding="UTF-8"?>\n<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Frameset//EN">\n',
  'x1.0b': '<?xml version="1.0" encoding="UTF-8"?>\n<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML Basic 1.0//EN">\n',
  'tagsoup': ''
}

## HTML- and URL-encoding

## encH

# Escape HTML-special characters. Uses &#xx; notation for characters except for &, <, >, " and hard-space.

encH_re= re.compile('[&<>"\x7F-\xFF\x01-\x1F]')
encH_special= {'&': '&amp;', '<': '&lt;', '>': '&gt;', '"': '&quot;', '\xA0': '&nbsp;'}

def encH_char(charMatch):
  x= charMatch.group(0)
  if encH_special.has_key(x):
    return encH_special[x]
  else:
    return '&#'+str(ord(x))+';'

def encH(text):
  return encH_re.sub(encH_char, text)

## encU

# Escape URL-special characters for inclusion in a GET string

encU_re= re.compile('[%<>#"{}\[\]|^`;?:@&=+,$\x01-\x20\x7F-\xFF]')

def encHex(x):
  if x<16:
    return '0'+'0123456789ABCDEF'[ord(x)]
  else:
    return '0123456789ABCDEF'[int(ord(x)/16)]+'0123456789ABCDEF'[int(ord(x)%16)]

def encU_char(charMatch):
  x= charMatch.group(0)
  return '%'+encHex(x)

def encU(text):
  return encU_re.sub(encU_char, text)

## encHU

# Encode as URL and then encode that as HTML, for convenience

def encHU(x):
  return encH(encU(x))

## queryString

# Encode dictionary of values into a GET query string

def queryString(dict):
  return '?'+string.join(map(lambda (k, v): encU(k)+'='+encU(v), dict.items()), '&')

## findIcon

# Given an icon specifier as returned by the Trait objects, attempt to find the best-fit icon file that actually exists on disc. Return the URL of it.

unsafe= re.compile('[^a-zA-Z0-9_\-]')

def makeSafe(x):
  return unsafe.sub('_', x)

def findIcon(spec):
  spec= ['icon']+map(makeSafe, map(string.lower, spec))
  while true:
    pth= config['imagePath']+os.sep+string.join(spec, os.sep)+iconType
    if (len(spec)<=1) or os.path.exists(pth):
      break
    spec[-1:]= []
  return (config['imageUrl']+string.join(spec, '/')+iconType, pth)

## Query serialisation
## =============================================================================

## encodeQuery

# Convert a query structure (a list of (trait, [values]) tuples) into a plain text
# representation in the form trait/subpart0//anothertrait/subpart0/subpart1
# '/' characters in values are escaped to '\_'; '\'s are escaped to '\\'; empty
# sections are escaped to '\ ' to avoid putting two '/'s together

def encodeQuery(query):
  return string.join(map(encodeCondition, query), '//')

def encodeCondition((trait, values)):
  return string.join(map(encodeValue, [trait]+stripRedundant(values)), '/')

def stripRedundant(values):
  try:
    p= values.index('')
  except ValueError:
    return values
  return values[:p+1]

def encodeValue(text):
  if text=='':
    return '\\.'
  else:
    return string.replace(string.replace(text, '\\', '\\\\'), '/', '\_')

## decodeQuery

# Convert a text-encoded query back to a query structure

def decodeQuery(text):
  if text=='':
    return []
  else:
    return map(decodeCondition, string.split(text, '//'))

def decodeCondition(text):
  condition= map(decodeValue, string.split(text, '/'))
  return (condition[0], condition[1:])

def decodeValue(text):
  return string.replace(string.replace(string.replace(text, '\\.', ''), '\\_', '/'), '\\\\', '\\')

